from typing import Any, Dict, List, Optional

from fuzzy_multi_dict import FuzzyMultiDict


class SpellChecker:
    """
    Simple Spell Checker is a spell checker based on prefix tree search.
    It find nearest to input word from known words (from input list).
    The algorithm finds mistakes in a word (insertions, deletions, replacements).

    """
    def __init__(
        self,
        max_mistakes_number: Optional[int] = 3,
        max_mistakes_number_part: Optional[float] = None,
    ):
        """
        :param Optional[int] max_mistakes_number: default value of maximum number
               of corrections in the query key when searching for a matching key
        :param Optional[float] max_mistakes_number_part: default value to calculate
               maximum number of corrections in the query key when searching
               for a matching dictionary key;
               calculated as round(max_mistakes_number_part * token_length)

        """
        self.__vocab = FuzzyMultiDict(
            max_mistakes_number=max_mistakes_number,
            max_mistakes_number_part=max_mistakes_number_part,
        )

    def add_words(self, words: List[str]):
        for word in set(words):

            if not isinstance(word, str):
                raise TypeError(f"Invalid word type; expect str; got {type(word)}")

            if not len(word):
                raise ValueError("Empty value")

            self.__vocab[word] = word

    def correction(
        self,
        word: str,
        max_mistakes_number: Optional[int] = None,
        max_mistakes_number_part: Optional[float] = None,
    ) -> List[Dict[str, Any]]:
        """
        :param word: word to correct
        :param int max_mistakes_number: maximum number of corrections in the word
        :param max_mistakes_number_part: value to calculate maximum number
               of corrections in the word;
               if not None - `max_mistakes_number` will be ignored;
               calculated as round(max_mistakes_number_part * word_length);

        :return List[Dict[str, Any]]: result in next format
            [
                {
                    'word': <corrected word>,
                    'corrections': <list of corrections>
                },
                ...
            ]

        """
        return [
            {"word": row["value"], "corrections": row["mistakes"]}
            for row in self.__vocab.get(
                key=word,
                max_mistakes_number=max_mistakes_number,
                max_mistakes_number_part=max_mistakes_number_part,
            )
        ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simple_spell_checker']

package_data = \
{'': ['*']}

install_requires = \
['fuzzy-multi-dict>=0.0.3,<0.0.4']

setup_kwargs = {
    'name': 'simple-spell-checker',
    'version': '0.0.1',
    'description': '',
    'long_description': '# simple_spell_checker\n\n[![Coverage Status](https://img.shields.io/badge/%20Python%20Versions-%3E%3D3.9-informational)](https://pypi.org/project/fuzzy_multi_dict/)\n[![Coverage Status](https://coveralls.io/repos/github/SemioTricks/fuzzy-multi-dict/badge.svg?branch=feature/initial)](https://coveralls.io/github/SemioTricks/fuzzy-multi-dict?branch=feature/initial)\n\n[![Coverage Status](https://img.shields.io/badge/Version-0.0.1-informational)](https://github.com/SemioTricks/simple-spell-checker)\n[![Coverage Status](https://img.shields.io/badge/Docs-passed-green)](https://github.com/SemioTricks/simple-spell-checker/tree/main/simple_spell_checker_doc)\n\n\nSimple Spell Checker is a spell checker based on prefix tree search. It find nearest to input word from known words (from input list). \nThe algorithm finds mistakes in a word (insertions, deletions, replacements).\n\n# Installation\n\n> pip install simple_spell_checker\n\n# Quickstart\n\n```python\nfrom simple_spell_checker.spell_checker import SpellChecker\n\ncities = [\n    "Kyiv", "Kharkiv", "Odesa", "Dnipro", "Donetsk", "Zaporizhzhia", "Lviv", \n    "Kryvyi Rih", "Mykolaiv", "Luhansk", "Vinnytsia", "Simferopol", "Chernihiv", \n    "Kherson", "Poltava", "Khmelnytskyi", "Cherkasy", "Chernivtsi", "Zhytomyr", "Sumy",\n    "Rivne", "Ivano-Frankivsk", "Ternopil", "Kropyvnytskyi", "Lutsk", "Uzhhorod"\n]\n\nspell_checker = SpellChecker(max_mistakes_number_part=.5)\nspell_checker.add_words(cities)\n\nspell_checker.correction(\'Kiev\')\n# [{\'word\': \'Kyiv\',\n#   \'corrections\': [{\'mistake_type\': \'missing symbol "y"\', \'position\': 1},\n#    {\'mistake_type\': \'extra symbol "e"\', \'position\': 2}]}]\n\nspell_checker.correction(\'odessa\')\n# [{\'word\': \'Odesa\',\n#   \'corrections\': [{\'mistake_type\': \'wrong symbol "o": replaced on "O"\',\n#     \'position\': 0},\n#    {\'mistake_type\': \'extra symbol "s"\', \'position\': 4}]}]\n\nspell_checker.correction(\'Hmelnitskiy\', max_mistakes_number_part=.5)\n# [{\'word\': \'Khmelnytskyi\',\n#   \'corrections\': [{\'mistake_type\': \'missing symbol "K"\', \'position\': 0},\n#    {\'mistake_type\': \'wrong symbol "H": replaced on "h"\', \'position\': 0},\n#    {\'mistake_type\': \'wrong symbol "i": replaced on "y"\', \'position\': 5},\n#    {\'mistake_type\': \'missing symbol "y"\', \'position\': 9},\n#    {\'mistake_type\': \'extra symbol "y"\', \'position\': 10}]}]\n\nspell_checker.correction(\'Kharkiv\')\n# [{\'word\': \'Kharkiv\', \'corrections\': []}]\n```',
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
